{#- used by abilian.web.frontend.DELETE_ACTION #}
<a class="{{ action.css_class }} js-widget" href="{{ url }}"
    data-init-with="deleteConfirm"
    data-init-params="{{ {
                      'title': 'La suppression est irréversible',
                      'message': _('Do you really want to delete this entity?'),
                      'label': _('Delete'),
                      'cancelLabel': _('Cancel'),
                      }|tojson|forceescape }}">
  {%- if action.icon %}{{ action.icon }} {% endif %}
  {{ action.title }}
</a>
