{%- if rows %}
  <table class="table table-condensed table-striped">
    {%- if labels %}
      <thead>
      <tr>
        {%- for label in labels %}
          <th>{{ label }}</th>
        {%- endfor %}
      </tr>
      </thead>
    {%- endif %}

    <tbody>
    {%- for row in rows %}
      <tr>
        {%- for value in row %}
          <td>{{ value }}</td>
        {%- endfor %}
      </tr>
    {%- endfor %}
    </tbody>
  </table>
{%- endif %}
